(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (factory((global.adblocker = {})));
}(this, (function (exports) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m) return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
        }
        catch (error) { e = { error: error }; }
        finally {
            try {
                if (r && !r.done && (m = i["return"])) m.call(i);
            }
            finally { if (e) throw e.error; }
        }
        return ar;
    }

    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }

    function getWindowHostname(window) {
        var strip = function (hostname) {
            if (hostname.startsWith('www.')) {
                return hostname.slice(4);
            }
            return hostname;
        };
        var win = window;
        while (win) {
            var hostname = win.location.hostname;
            if (hostname !== '') {
                return strip(hostname);
            }
            if (win === window.parent) {
                break;
            }
            win = win.parent;
        }
        return '';
    }
    var magic = Math.abs((Date.now() * 524287) ^ ((Math.random() * 524287) >>> 0)).toString(16);

    function autoRemoveScript(script) {
        return "\ntry {\n  " + script + "\n} catch (ex) { }\n\n(function() {\n  var currentScript = document.currentScript;\n  var parent = currentScript && currentScript.parentNode;\n\n  if (parent) {\n    parent.removeChild(currentScript);\n  }\n})();\n  ";
    }
    function wrapCallableInContext(script, deps) {
        if (deps === void 0) { deps = []; }
        return "\n" + deps.map(function (dep) { return "const " + dep.name + " = " + dep.toString() + ";"; }).join('\n') + "\n" + script + "\n  ";
    }
    function autoCallFunction(fn) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        return "\ntry {\n  (" + fn.toString() + ")(" + args.map(function (arg) { return JSON.stringify(arg); }).join(', ') + ");\n} catch (ex) {  };";
    }
    function blockScript(filter, doc) {
        var filterRE = new RegExp(filter);
        doc.addEventListener('beforescriptexecute', function (ev) {
            var target = ev.target;
            if (target.textContent && filterRE.test(target.textContent)) {
                ev.preventDefault();
                ev.stopPropagation();
            }
        });
    }
    function injectCSSRule(rule, doc) {
        var css = doc.createElement('style');
        css.type = 'text/css';
        css.id = 'cliqz-adblokcer-css-rules';
        var parent = doc.head || doc.documentElement;
        if (parent !== null) {
            parent.appendChild(css);
            css.appendChild(doc.createTextNode(rule));
        }
    }
    function injectScript(s, doc) {
        var script = doc.createElement('script');
        script.type = 'text/javascript';
        script.id = 'cliqz-adblocker-script';
        script.async = false;
        script.appendChild(doc.createTextNode(autoRemoveScript(s)));
        var parent = doc.head || doc.documentElement;
        if (parent !== null) {
            parent.appendChild(script);
        }
    }
    function bundle(fn, deps) {
        if (deps === void 0) { deps = []; }
        return function (window) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            return injectScript(wrapCallableInContext(autoCallFunction.apply(void 0, __spread([fn], args)), deps), window.document);
        };
    }

    var swallowOwnErrors = bundle(function (magic) {
        var windowOnError = window.onerror;
        var customOnError = function (msg) {
            if (typeof msg === 'string' && msg.indexOf(magic) !== -1) {
                return true;
            }
            if (windowOnError instanceof Function) {
                return windowOnError.apply(this, arguments);
            }
            return false;
        }.bind(window);
        Object.defineProperty(window, 'onerror', {
            get: function () { return customOnError; },
            set: function () {
            }
        });
    });
    var protectConsole = bundle(function () {
        var originalConsole = window.console;
        var originalLog = console.log;
        Object.defineProperty(console, 'log', {
            value: function fakeLog() {
                for (var i = 0; i < arguments.length; i++) {
                    if (arguments[i] instanceof HTMLElement) {
                        return;
                    }
                }
                return originalLog.apply(originalConsole, arguments);
            }.bind(console)
        });
        Object.defineProperty(console.log, 'name', { value: 'log' });
    });

    function isCurrentScriptInstart(thisScript) {
        var script = document.currentScript;
        if (script === thisScript) {
            return false;
        }
        if (script instanceof HTMLScriptElement) {
            var src = script.src;
            if (src.indexOf('instart.js') !== -1 || src.indexOf('?:i10c.') !== -1) {
                return true;
            }
            var scriptContent = script.textContent;
            if (scriptContent !== null &&
                (scriptContent.indexOf(':Instart-') !== -1 ||
                    scriptContent.indexOf('I10C') !== -1 ||
                    scriptContent.indexOf('IXC') !== -1 ||
                    scriptContent.indexOf('INSTART') !== -1)) {
                return true;
            }
        }
        return false;
    }
    var shieldPropertiesFromInstart = bundle(function (magic$$1) {
        var thisScript = document.currentScript;
        [
            'atob',
            'console.error',
            'INSTART_TARGET_NAME',
            'navigator.userAgent',
            'performance',
            'require',
        ].forEach(function (target) {
            var obj = window;
            var chain = target.split('.');
            var prop = chain.pop();
            chain.forEach(function (part) {
                obj = obj[part];
            });
            if (prop === undefined || obj === undefined) {
                return;
            }
            Object.defineProperty(obj, prop, (function () {
                var value = obj[prop];
                return {
                    get: function () {
                        if (isCurrentScriptInstart(thisScript)) {
                            throw new ReferenceError(magic$$1);
                        }
                        return value;
                    },
                    set: function (a) {
                        if (isCurrentScriptInstart(thisScript)) {
                            throw new ReferenceError(magic$$1);
                        }
                        value = a;
                    }
                };
            })());
        });
    }, [isCurrentScriptInstart]);
    var monkeyPatchNanoVisor = bundle(function (magic$$1) {
        var HtmlStreamingMock = {
            InsertTags: function (_, b) {
                document.write(b);
            },
            MoveTagAndCleanUp: function () {
            },
            MoveTag: function () {
            },
            RemoveTags: function () {
            },
            InterceptNode: function () {
            },
            PatchBegin: function () {
            },
            PatchEnd: function () {
            },
            PatchInit: function () {
            },
            PatchK: function () {
            },
            ReloadWithNoHtmlStreaming: function () {
                window.location.reload(true);
            },
            RemoveAttributes: function () {
            },
            UpdateAttributes: function () {
            },
            SendR: function () {
            },
            RemoveCurrentScript: function () {
            }
        };
        var nanoVisorProxy = new Proxy({}, {
            get: function (target, name) {
                switch (name) {
                    case 'HtmlStreaming':
                        return HtmlStreamingMock;
                    default: {
                        return target[name];
                    }
                }
            },
            set: function (target, name, value) {
                switch (name) {
                    case 'CanRun':
                        target.CanRun = function () { return false; };
                        break;
                    default:
                        target[name] = value;
                }
                return true;
            }
        });
        var instartInit;
        Object.defineProperty(window, '_bcm_il', { value: true });
        Object.defineProperty(window, 'I10C', { value: nanoVisorProxy });
        Object.defineProperty(window, 'I11C', { value: nanoVisorProxy });
        Object.defineProperty(window, 'INSTART', {
            value: new Proxy({}, {
                get: function (target, name) {
                    switch (name) {
                        case 'Init':
                            return function (a) {
                                if (a instanceof Object &&
                                    typeof a.nanovisorGlobalNameSpace === 'string' &&
                                    a.nanovisorGlobalNameSpace !== '') {
                                    window[a.nanovisorGlobalNameSpace] = nanoVisorProxy;
                                }
                                a.disableInjectionXhr = true;
                                a.enableHtmlStreaming = false;
                                a.enableQSCallDiffComputationConfig = false;
                                a.enableQuerySelectorMonitoring = false;
                                a.partialImage = false;
                                a.rum = false;
                                a.serveNanovisorSameDomain = false;
                                a.useWrapper = false;
                                a.virtualDomains = 0;
                                a.virtualizeDomains = [];
                                if (instartInit !== undefined) {
                                    instartInit(a);
                                }
                            };
                        default:
                            if (target[name] === undefined) {
                                throw new Error(magic$$1);
                            }
                            return target[name];
                    }
                },
                set: function (target, name, value) {
                    switch (name) {
                        case 'Init':
                            instartInit = value;
                            break;
                        default:
                            target[name] = value;
                    }
                    return true;
                }
            })
        });
    });
    function instart(window) {
        if ([
            'afterellen.com',
            'allakhazam.com',
            'americanphotomag.com',
            'atvrider.com',
            'baggersmag.com',
            'baltimoresun.com',
            'boatingmag.com',
            'boston.com',
            'cafemom.com',
            'calgaryherald.com',
            'calgarysun.com',
            'capitalgazette.com',
            'carrollcountytimes.com',
            'cattime.com',
            'cbssports.com',
            'celebslam.com',
            'celebuzz.com',
            'chicagotribune.com',
            'chowhound.com',
            'chron.com',
            'chroniclelive.co.uk',
            'citypaper.com',
            'cnet.com',
            'comingsoon.net',
            'computershopper.com',
            'courant.com',
            'craveonline.com',
            'cruisingworld.com',
            'csgoutpost.com',
            'ctnow.com',
            'cycleworld.com',
            'dailydot.com',
            'dailypress.com',
            'dayzdb.com',
            'deathandtaxesmag.com',
            'delmartimes.net',
            'destinationweddingmag.com',
            'dirtrider.com',
            'diversitybestpractices.com',
            'dogtime.com',
            'dotaoutpost.com',
            'download.cnet.com',
            'edmontonjournal.com',
            'edmontonsun.com',
            'edmunds.com',
            'emedicinehealth.com',
            'esohead.com',
            'everquest.allakhazam.com',
            'everydayhealth.com',
            'extremetech.com',
            'fieldandstream.com',
            'financialpost.com',
            'floridatravellife.com',
            'flyingmag.com',
            'focus.de',
            'gamepedia.com',
            'gamerevolution.com',
            'gamespot.com',
            'geek.com',
            'gofugyourself.com',
            'growthspotter.com',
            'hearthhead.com',
            'hockeysfuture.com',
            'hotbikeweb.com',
            'hoylosangeles.com',
            'ibtimes.com',
            'idigitaltimes.com',
            'ign.com',
            'infinitiev.com',
            'islands.com',
            'lajollalight.com',
            'laptopmag.com',
            'latintimes.com',
            'leaderpost.com',
            'legacy.com',
            'lifewire.com',
            'livescience.com',
            'lolking.net',
            'mamaslatinas.com',
            'marlinmag.com',
            'mcall.com',
            'medicaldaily.com',
            'medicinenet.com',
            'metacritic.com',
            'metrolyrics.com',
            'mmo-champion.com',
            'momtastic.com',
            'montrealgazette.com',
            'motorcyclecruiser.com',
            'motorcyclistonline.com',
            'motortrend.com',
            'msn.com',
            'musicfeeds.com.au',
            'mustangandfords.com',
            'mysanantonio.com',
            'nasdaq.com',
            'nationalpost.com',
            'newsarama.com',
            'newsweek.com',
            'opshead.com',
            'orlandosentinel.com',
            'ottawacitizen.com',
            'ottawasun.com',
            'outdoorlife.com',
            'pcmag.com',
            'playstationlifestyle.net',
            'popphoto.com',
            'popsci.com',
            'ranchosantafereview.com',
            'range365.com',
            'ranker.com',
            'realclearpolitics.com',
            'realitytea.com',
            'redeyechicago.com',
            'salon.com',
            'saltwatersportsman.com',
            'sandiegouniontribune.com',
            'saveur.com',
            'scubadiving.com',
            'scubadivingintro.com',
            'seattlepi.com',
            'sfgate.com',
            'sherdog.com',
            'slate.com',
            'slickdeals.net',
            'southflorida.com',
            'space.com',
            'spin.com',
            'sporcle.com',
            'sportdiver.com',
            'sportfishingmag.com',
            'sportingnews.com',
            'sportrider.com',
            'spox.com',
            'stereogum.com',
            'streetchopperweb.com',
            'sun-sentinel.com',
            'superherohype.com',
            'superstreetbike.com',
            'tenplay.com.au',
            'tf2outpost.com',
            'thebalance.com',
            'thefashionspot.com',
            'thefrisky.com',
            'theprovince.com',
            'thespruce.com',
            'thestarphoenix.com',
            'thesuperficial.com',
            'thoughtcatalog.com',
            'thoughtco.com',
            'timeanddate.com',
            'timesunion.com',
            'tmn.today',
            'tomsguide.com',
            'tomsguide.fr',
            'tomshardware.co.uk',
            'tomshardware.com',
            'tomshardware.de',
            'tomshardware.fr',
            'torontosun.com',
            'totalbeauty.com',
            'trustedreviews.com',
            'tv.com',
            'tvguide.com',
            'tvtropes.org',
            'twincities.com',
            'utvdriver.com',
            'vancouversun.com',
            'vg.no',
            'vibe.com',
            'wakeboardingmag.com',
            'washingtonpost.com',
            'waterskimag.com',
            'webmd.com',
            'wikia.com',
            'windowscentral.com',
            'windsorstar.com',
            'winnipegsun.com',
            'workingmother.com',
            'wowhead.com',
            'wrestlezone.com',
            'xda-developers.com',
            'yachtingmagazine.com',
            'zam.com',
        ].indexOf(getWindowHostname(window)) !== -1) {
            swallowOwnErrors(window, magic);
            monkeyPatchNanoVisor(window, magic);
            shieldPropertiesFromInstart(window, magic);
        }
    }

    var injectCircumvention = (function (window) {
        instart(window);
    });

    var CosmeticInjection = (function () {
        function CosmeticInjection(window, backgroundAction, useMutationObserver) {
            if (useMutationObserver === void 0) { useMutationObserver = true; }
            this.window = window;
            this.backgroundAction = backgroundAction;
            this.mutationObserver = null;
            this.injectedRules = new Set();
            this.injectedScripts = new Set();
            this.blockedScripts = new Set();
            this.observedNodes = new Set();
            this.backgroundAction('getCosmeticsForDomain');
            if (useMutationObserver) {
                this.onMutation([{ target: this.window.document.body }]);
                this.startObserving();
            }
        }
        CosmeticInjection.prototype.unload = function () {
            if (this.mutationObserver) {
                try {
                    this.mutationObserver.disconnect();
                }
                catch (e) {
                }
            }
        };
        CosmeticInjection.prototype.injectCircumvention = function () {
            injectCircumvention(this.window);
        };
        CosmeticInjection.prototype.handleResponseFromBackground = function (_a) {
            var active = _a.active, scripts = _a.scripts, blockedScripts = _a.blockedScripts, styles = _a.styles;
            if (!active) {
                this.unload();
                return;
            }
            for (var i = 0; i < scripts.length; i += 1) {
                var script = scripts[i];
                if (!this.injectedScripts.has(script)) {
                    injectScript(script, this.window.document);
                    this.injectedScripts.add(script);
                }
            }
            for (var i = 0; i < blockedScripts.length; i += 1) {
                var script = blockedScripts[i];
                if (!this.blockedScripts.has(script)) {
                    blockScript(script, this.window.document);
                    this.blockedScripts.add(script);
                }
            }
            this.handleRules(styles);
        };
        CosmeticInjection.prototype.handleRules = function (rules) {
            var rulesToInject = [];
            for (var i = 0; i < rules.length; i += 1) {
                var rule = rules[i];
                if (!this.injectedRules.has(rule)) {
                    try {
                        if (!this.window.document.querySelector(rule)) {
                            continue;
                        }
                    }
                    catch (e) {
                        continue;
                    }
                    this.injectedRules.add(rule);
                    rulesToInject.push(" :root " + rule);
                }
            }
            if (rulesToInject.length > 0) {
                injectCSSRule(rulesToInject.join(' ,') + " {display:none !important;}", this.window.document);
            }
        };
        CosmeticInjection.prototype.onMutation = function (mutations) {
            var _this = this;
            var targets = new Set(mutations.map(function (m) { return m.target; }).filter(function (t) { return t; }));
            if (targets.size > 100) {
                targets = new Set([this.window.document.body]);
            }
            if (targets.size === 0) {
                return;
            }
            var nodeInfo = new Set();
            targets.forEach(function (target) {
                var nodes = target.querySelectorAll('*');
                for (var i = 0; i < nodes.length; i += 1) {
                    var node = nodes[i];
                    if (node.hidden) {
                        continue;
                    }
                    if (node.id) {
                        var selector = "#" + node.id;
                        if (!_this.observedNodes.has(selector)) {
                            nodeInfo.add(selector);
                            _this.observedNodes.add(selector);
                        }
                    }
                    if (node.tagName) {
                        var selector = node.tagName;
                        if (!_this.observedNodes.has(selector)) {
                            nodeInfo.add(selector);
                            _this.observedNodes.add(selector);
                        }
                    }
                    if (node.className && node.className.split) {
                        node.className.split(' ').forEach(function (name) {
                            var selector = "." + name;
                            if (!_this.observedNodes.has(selector)) {
                                nodeInfo.add(selector);
                                _this.observedNodes.add(selector);
                            }
                        });
                    }
                }
            });
            if (nodeInfo.size > 0) {
                this.backgroundAction('getCosmeticsForNodes', [__spread(nodeInfo)]);
            }
        };
        CosmeticInjection.prototype.startObserving = function () {
            var _this = this;
            if (this.window.MutationObserver !== undefined) {
                this.mutationObserver = new this.window.MutationObserver(function (mutations) {
                    return _this.onMutation(mutations);
                });
                this.mutationObserver.observe(this.window.document, {
                    childList: true,
                    subtree: true
                });
            }
        };
        return CosmeticInjection;
    }());

    exports.CosmeticsInjection = CosmeticInjection;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
